-- **************************************************************************
--  This is the 32 times CIC over sampling filter(I axis)
--				2013,12,28 by Y nishimura
-- **************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
LIBRARY lpm;
use lpm.lpm_components.all;

entity OverCICi is
	port
		(
			Hclk		: in  std_logic;	-- Master 4fsc clock
			Reset		: in  std_logic;	-- Master Reset
			CCIrst		: in  std_logic;
			IBuf		: in  std_logic_vector(17 downto 0); -- I/Q data
			Rtim		: in  std_logic;	-- Base timing = 1GHz/248/128*4 (4 times)
			Hsamp		: in  std_logic;	-- High rate = 1GHz/248
			DaOut		: out std_logic_vector(17 downto 0)  -- Comb filter out
		);
end OverCICi;

architecture comb of OverCICi is
	signal	MemoO,InteE			: std_logic_vector(35 downto 0);
	signal	DifE				: std_logic_vector(35 downto 0);
	signal	InteA,InteB,InteC	: std_logic_vector(32 downto 0);
	signal	InteD				: std_logic_vector(32 downto 0);
	signal	DifD				: std_logic_vector(21 downto 0);
	signal	DifC,Inbud			: std_logic_vector(20 downto 0);
	signal	DifB,Inbuc			: std_logic_vector(19 downto 0);
	signal	DifA,Inbub			: std_logic_vector(18 downto 0);
	signal	Inbua,Dout			: std_logic_vector(17 downto 0);
	signal	Shape				: std_logic_Vector(9 downto 0);
	signal	Wadd				: std_logic_vector(4 downto 0);
	signal	ShapeA				: std_logic_vector(1 downto 0);
	signal	Ramen				: std_logic;
	
begin
	DaOut <= Dout;
-------------------------------------------------------------------------------
--	1st Differential
-------------------------------------------------------------------------------
  	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(18 downto 0);
		
	begin
		if Reset = '0' then
			DifA  <= "0000000000000000000"; Inbua <= "000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				DifA  <= "0000000000000000000"; Inbua <= "000000000000000000";
			else
				Tempa(18) := Inbua(17); Tempa(17 downto 0) := Inbua;
				Tempb(18) := IBuf(17);  Tempb(17 downto 0) := IBuf;
				if Rtim = '1'		then DifA <= Tempb - Tempa; Inbua <= IBuf;
									else DifA <= DifA; Inbua <= Inbua;
				end if;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--	2nd Differential
-------------------------------------------------------------------------------
  	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(19 downto 0);
		
	begin
		if Reset = '0' then
			DifB  <= "00000000000000000000"; Inbub <= "0000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				DifB  <= "00000000000000000000"; Inbub <= "0000000000000000000";
			else
				Tempa(19) := Inbub(18); Tempa(18 downto 0) := Inbub;
				Tempb(19) := DifA(18);  Tempb(18 downto 0) := DifA;
				if Rtim = '1'		then DifB <= Tempb - Tempa; Inbub <= DifA;
									else DifB <= DifB; Inbub <= Inbub;
				end if;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--	3rd Differential
-------------------------------------------------------------------------------
  	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(20 downto 0);
		
	begin
		if Reset = '0' then
			DifC  <= "000000000000000000000"; Inbuc <= "00000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				DifC  <= "000000000000000000000"; Inbuc <= "00000000000000000000";
			else
				Tempa(20) := Inbuc(19); Tempa(19 downto 0) := Inbuc;
				Tempb(20) := DifB(19);  Tempb(19 downto 0) := DifB;
				if Rtim = '1'		then DifC <= Tempb - Tempa; Inbuc <= DifB;
									else DifC <= DifC; Inbuc <= Inbuc;
				end if;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--	4th Differential
-------------------------------------------------------------------------------
  	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(21 downto 0);
		
	begin
		if Reset = '0' then
			DifD  <= "0000000000000000000000"; Inbud <= "000000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				DifD  <= "0000000000000000000000"; Inbud <= "000000000000000000000";
			else
				Tempa(21) := Inbud(20); Tempa(20 downto 0) := Inbud;
				Tempb(21) := DifC(20);  Tempb(20 downto 0) := DifC;
				if Rtim = '1'		then DifD <= Tempb - Tempa; Inbud <= DifC;
									else DifD <= DifD; Inbud <= Inbud;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		1st Integral
--------------------------------------------------------------------------------
 	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(32 downto 0);
		
	begin
		if Reset = '0' then 
			InteA <= "000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				InteA <= "000000000000000000000000000000000";
			else
				if DifD(21) = '1'	then Tempa(32 downto 22) := "11111111111";
									else Tempa(32 downto 22) := "00000000000";
				end if;
				Tempa(21 downto 0) := DifD;
				if Rtim = '1'	then InteA <= InteA + Tempa;
								else InteA <= InteA;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		2nd Integral
--------------------------------------------------------------------------------
 	process(Hclk,Reset)
	begin
		if Reset = '0' then 
			InteB <= "000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				InteB <= "000000000000000000000000000000000";
			else
				if Hsamp = '1'	then InteB <= InteB + InteA;
								else InteB <= InteB;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		3rd Integral
--------------------------------------------------------------------------------
 	process(Hclk,Reset)
	begin
		if Reset = '0' then 
			InteC <= "000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				InteC <= "000000000000000000000000000000000";
			else
				if Hsamp = '1'	then InteC <= InteC + InteB;
								else InteC <= InteC;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		4th Integral
--------------------------------------------------------------------------------
 	process(Hclk,Reset)
	begin
		if Reset = '0' then 
			InteD <= "000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				InteD <= "000000000000000000000000000000000";
			else
				if Hsamp = '1'	then InteD <= InteD + InteC;
								else InteD <= InteD;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		5th Integral
--------------------------------------------------------------------------------
	process(Hclk,Reset)
		variable Tempa : std_logic_Vector(35 downto 0);
		variable Tempb,Tempc,Tempd : std_logic_vector(2 downto 0);
	begin
		if Reset = '0' then 
			InteE <= "000000000000000000000000000000000000"; ShapeA <= "00";
		elsif rising_edge(Hclk) then
			Tempb(2) := '0'; Tempb(1 downto 0) := InteD(1 downto 0);
			Tempc(2) := '0'; Tempc(1 downto 0) := ShapeA;
			Tempd := Tempc + Tempd;
			if CCIrst = '0'	then
				InteE <= "000000000000000000000000000000000000"; ShapeA <= "00";
			else
				if InteD(32) = '1'	then Tempa(35 downto 31) := "11111";
									else Tempa(35 downto 31) := "00000";
				end if;
				Tempa(30 downto 0) := InteD(32 downto 2);
				if Hsamp = '1'	then
					ShapeA <= Tempd(1 downto 0);
					if Tempd(2) = '1'	then InteE <= InteE + Tempa + 1;
										else InteE <= InteE + Tempa;
					end if;
				else InteE <= InteE; ShapeA <= ShapeA;
				end if;
			end if;
		end if;
	end process;

--------------------------------------------------------------------------------
--		5th Differential
--------------------------------------------------------------------------------
	Ramen <= Hsamp or (not Reset);
	process(Hclk)
	begin
		if rising_edge(Hclk) then
			if Ramen = '1'	then Wadd <= Wadd + 1;
							else Wadd <= Wadd;
			end if;
		end if;
	end process;

	U1: lpm_ram_dp
   		GENERIC map (
						LPM_WIDTH =>36, LPM_WIDTHAD => 5,
      					LPM_OUTDATA => "REGISTERED")
		port map (
					data => InteE,
					wraddress => Wadd,
					wrclken => '1',
					wren => Ramen,
					wrclock => Hclk,
					rdaddress => Wadd,
					rden => '1',
					rdclken => '1',
					rdclock => Hclk,
					q => MemoO
				  );

  	process(Hclk,Reset)
		variable Tempc,Tempd,Tempe : std_logic_vector(10 downto 0);
		
	begin
		if Reset = '0' then
			DifE <= "000000000000000000000000000000000000"; Shape <= "0000000000";
			Dout <= "000000000000000000";
		elsif rising_edge(Hclk) then
			if CCIrst = '0'	then
				DifE <= "000000000000000000000000000000000000"; Shape <= "0000000000";
				Dout <= "000000000000000000";
			else
				if Hsamp = '1'		then DifE <= InteE - MemoO;
									else DifE <= DifE;
				end if;
				Tempc(10) := '0'; Tempc(9 downto 0) := DifE(17 downto 8);
				Tempd(10) := '0'; Tempd(9 downto 0) := Shape;
				Tempe := Tempc + Tempd;
				if Hsamp = '1'		then
					if Tempe(10) = '1'	then Dout <= DifE(35 downto 18) + 1;
										else Dout <= DifE(35 downto 18);
					end if;
					Shape <= Tempe(9 downto 0);
				else Dout <= Dout; Shape <= Shape;
				end if;
			end if;
		end if;
	end process;


end comb;

